///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////
//             Installer             //
////////////////////////////////////////////////////////

unit InstallerServiceClass;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, forms, Registry, ExtraFunctionsLcl, GGBuildToolsShared, GGConsUtilServiceClass, ExtraFileUtilsLCL;
  
const
  GGBT_InstallerAppName = 'Installer';
  
type
  TInstallerServiceClass = class (TGGConsUtilServiceClass)       //      
  private
  protected
  public
    //
    Registr : TRegistry;               //      .
    InstalledTools : RInstalledTools;  //   
    
    //-...
    constructor Create; override;
    destructor Destroy; override;
    
    // ...
    procedure DoWork;        override;   //   %).
    procedure InstallIt;                 // .
    procedure ShowDescription; override; //  .
    
  end;
  
var
  InstServClass : TInstallerServiceClass;

implementation

/////////////////////////////////////////////
//         TInstallerServiceClass          //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

constructor TInstallerServiceClass.Create;
begin
  //  ...
  InstalledTools.IssBuilder_installed := false;
  InstalledTools.StarterModBuilder_installed := false;
  InstalledTools.StarterModSynchronizator_installed := false;
  InstalledTools.VerRevUpdater_installed := false;
  
  //   ...
  Registr := TRegistry.Create;
  
  //  .
  Self.GGConsAppName := GGBT_InstallerAppName;
  
  //  ...
  inherited;
end;

destructor TInstallerServiceClass.Destroy;
begin
  // 
  Registr.Free;

  //  
  inherited;
end;

//------------------------------------------//
//              ...             //
//------------------------------------------//

procedure TInstallerServiceClass.DoWork;
//   %).
begin
  //     -    ProcessFurther := false
  //    .
  
  //  
  inherited;
  
  //       -    ....
  if ProcessFurther = true then begin
    //  
    ProcessFurther := false;
    //....
    InstallIt;
  end;
  
end;

procedure TInstallerServiceClass.InstallIt;
var
  CurPath, CurVer, ExePath, OldPath, Answer : string;
  NeedInstall, WasOldPath, OldVersionInstalled : boolean;
begin
  // .
  
  //  .    ,    -     ,
  //   -    ,   PATH,  
  //   - .          .
  
  NeedInstall := false;   //  -     %).
  WasOldPath := false;   //        .
  ExePath := DelLastSlash(ExtractFilePath(Application.ExeName))+'\bin';   //    GGBuildTools.
  OldVersionInstalled := false;   //      .
  
  //  ...
  ShowVersion();
  // .
  Writeln();
  
  //   ...
  Writeln('Gathering infirmation...');
  Writeln('');
  
  //,   ...
  Registr.RootKey := HKEY_CURRENT_USER;
  if Registr.OpenKey('\Software\Gipat Group\'+AllProjectName,false) = true then begin
  //if Registr.KeyExists('\Software\Gipat Group\'+AllProjectName,false) = true then begin    //#LCL-BUG
    //Registr.OpenKey('\Software\Gipat Group\'+AllProjectName,false);                        //#LCL-BUG
    
    //  ...     %).
    CurPath := Registr.ReadString('path');
    CurVer := Registr.ReadString('ver');
    Registr.CloseKey;
    
    //.  - ,    ...
    if UpperCase(DelLastSlash(ExtractFilePath(Application.ExeName))) <> UpperCase(DelLastSlash(CurPath)) then begin
      //,  .
      WasOldPath := true;
      OldPath := DelLastSlash(CurPath);
    end;
    
    //    ...
    if IsNewVersion(CurVer,Self.AppVersion.GenerateDotsVersionString()) = true then begin
      //   ...
      OldVersionInstalled := true;
    end;
    
    //    ,     -     
    //,     %).
    if (WasOldPath = false) and (OldVersionInstalled = false) then begin
      Writeln('This version is already installed into this directory.');
      Writeln('So, we no need to reinstall now.');
      Writeln('Installation aborted %)');
    end;
    
    //     ...
    
    //    .
    If WasOldPath = true then begin
      //  .
      if DoSilent <> true then begin
        Writeln('Another version is installed with path:');
        Writeln(OldPath);
        Write('Continue installing this version, deinstalling this another? (y/n) > ');
        Readln(Answer);
      end
      else begin
        Writeln('Another version is installed with path:');
        Writeln(OldPath);
        Writeln('I''ll deinstall this another!');
        Answer := 'y';
      end;
      
      if UpperCase(Answer) = 'Y' then begin
        NeedInstall := true;  //  .
      end;
    end;

    //    ,      .
    If (WasOldPath = false) and (OldVersionInstalled = true) then begin
      //  (  %) )     .
      if DoSilent <> true then begin
        Writeln('Older version ( '+CurVer+' ) is installed.');
        Write('Update (y) or cancel (n) ? (y/n) > ');
        Readln(Answer);
      end
      else begin
        Writeln('Older version ( '+CurVer+' ) is installed.');
        Writeln('Updating!');
        Answer := 'y';
      end;
      
      if UpperCase(Answer) = 'Y' then begin
        NeedInstall := true;  //  .
      end;
    end;
    
  end
  else begin
    //   .  ...
    NeedInstall := true;
  end;
  
  //    %)
  if NeedInstall = true then begin
    if DoSilent = false then begin
      Write('All is ready. Really want to install? (y/n) > ');
      Readln(Answer);
      if UpperCase(Answer) <> 'Y' then begin
        NeedInstall := false;  //,  .
        Writeln('Ok, canceled.');
      end
      else begin
        Writeln('Ok, installing is confirmed.');
      end;
    end
    else begin
      Writeln('All is ready. Installing is confirmed automatically.');
    end;
  end;
  
  //  .
  if NeedInstall = true then begin
  
    Writeln('');
    Writeln('');
    Writeln('');
    Writeln('Installing started!');
    Writeln('');
    Writeln('');

///////////////////////////

    //  ...
    Writeln('Checking tools: ');

    Write('IssBuilder                              ');
    if FileExists(DelLastSlash(ExePath)+'\IssBuilder.exe') = true then begin
      // ...
      InstalledTools.IssBuilder_installed := true;
      Writeln('[  OK  ]');
    end
    else begin
      // .
      InstalledTools.IssBuilder_installed := false;
      Writeln('[ FAIL ]');
    end;
    
    Write('VerRevUpdater                           ');
    if FileExists(DelLastSlash(ExePath)+'\VerRevUpdater.exe') = true then begin
      // ...
      InstalledTools.VerRevUpdater_installed := true;
      Writeln('[  OK  ]');
    end
    else begin
      // .
      InstalledTools.VerRevUpdater_installed := false;
      Writeln('[ FAIL ]');
    end;
    
    Write('StarterModBuilder                       ');
    if FileExists(DelLastSlash(ExePath)+'\StarterModBuilder.exe') = true then begin
      // ...
      InstalledTools.StarterModBuilder_installed := true;
      Writeln('[  OK  ]');
    end
    else begin
      // .
      InstalledTools.StarterModBuilder_installed := false;
      Writeln('[ FAIL ]');
    end;
    
    Write('StarterModSynchtonizator                ');
    if FileExists(DelLastSlash(ExePath)+'\StarterModSynchronizator.exe') = true then begin
      // ...
      InstalledTools.StarterModSynchronizator_installed := true;
      Writeln('[  OK  ]');
    end
    else begin
      // .
      InstalledTools.StarterModSynchronizator_installed := false;
      Writeln('[ FAIL ]');
    end;
    
    Write('SourcePackBuilder                       ');
    if FileExists(DelLastSlash(ExePath)+'\SourcePackBuilder.exe') = true then begin
      // ...
      InstalledTools.SourcePackBuilder_installed := true;
      Writeln('[  OK  ]');
    end
    else begin
      // .
      InstalledTools.SourcePackBuilder_installed := false;
      Writeln('[ FAIL ]');
    end;
    
    Writeln('');
    
/////////////////////////////

    // -     ...
    if WasOldPath = true then begin
      //   .
      if EnvPathRemove(OldPath,true,true) = true then begin
        Writeln('Deleted old path from environment.');
      end;
    end;
    
    // -   -    ...
    if Registr.OpenKey('\Software\Gipat Group\'+AllProjectName,false) = true then begin
      Registr.CloseKey();
      Registr.DeleteKey('\Software\Gipat Group\'+AllProjectName);
      Writeln('Deleted old registry key.');
    end;
    
    // -   .
    Writeln('');
    Writeln('Standard installation started.');
    Writeln('');
    //   User path-.
    Write('Adding into PATH wariable: ');
    EnvPathAdd(DelLastSlash(ExePath),EnvPathAdd_User);
    Writeln(DelLastSlash(ExePath));
    Writeln('done.');
    Writeln('');
    //   .
    Writeln('Registering in a registry...');
    Registr.RootKey := HKEY_CURRENT_USER;
    Registr.OpenKey('\Software\Gipat Group\'+AllProjectName, true);
    //    ...
    Registr.WriteString('path',DelLastSlash(ExtractFilePath(Application.ExeName)));
    Registr.WriteString('ver',Self.AppVersion.GenerateDotsVersionString());
    Registr.WriteBool('IssBuilder_installed',InstalledTools.IssBuilder_installed);
    Registr.WriteBool('VerRevUpdater_installed',InstalledTools.VerRevUpdater_installed);
    Registr.WriteBool('StarterModBuilder_installed',InstalledTools.StarterModBuilder_installed);
    Registr.WriteBool('StarterModSynchronizator_installed',InstalledTools.StarterModSynchronizator_installed);
    Registr.WriteBool('SourcePackBuilder_installed',InstalledTools.SourcePackBuilder_installed);
    // ...
    Registr.CloseKey;
    Writeln('done.');
    Writeln('');
    //    .
    Writeln('Installation completed succesfully!');
    Writeln('');
  end;
  
end;

procedure TInstallerServiceClass.ShowDescription;
//  .
begin
  inherited;
  Writeln('Description:');
  Writeln('             Installs '+AllProjectName+' to your system. No files copying, only');
  Writeln('             registering in windows registry and environment variables.');
end;


/////////////////////////////////
initialization

// %).  ,   ...     %)
begin
  InstServClass := TInstallerServiceClass.Create;
end;

finalization
//  ...
begin
  InstServClass.Free;
end;

end.

