///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
//            (..            //
//            )   IssBuilder         //
///////////////////////////////////////////////////////////////

//    InnoSetuo 5.

unit IssBuilderLogic; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, SconfFormat, IssFormat, VerInfoTxtFormat, ExtraFileUtilsLCL,
  Registry, FlistFormat, process, forms, FileUtil, ExtraFunctionsLCL;
  
const
  ISSB_CompilerExecutableName = 'ISCC.exe';

type
  FPostExternalMessage = procedure(const TextMessage : AnsiString) of object;    //       .

  TIssBuilderLogic = class     //,    ,     
  private
     Registr : TRegistry;               //      .
  protected
    // .
    
  public
    //.
    ErrorMessage : AnsiString;                          //  .
    PostExternalMessage : FPostExternalMessage;         //,     .
    
    //  " ".
    SconfFilePath : AnsiString;         //    sconf-.
    ProjectName : AnsiString;           // .
    ProjectLocalRoot : AnsiString;      //   .
    ProjectGlobalRoot : AnsiString;     //   .
    ProjectSconf : TSconfFormat;        //   sconf .
    ProjectVit : TVerInfoTxtFormat;     //  \ *.vit
    DefaultIss : TIssFormat;            //    *.iss
    CurrentIss : TIssFormat;            //  iss-.
    CurrentFlist : TFlistFormat;        //    .
    CurrentIssFileName : AnsiString;    //      iss.
    NoCacheWrite : Boolean;             // true -        .   false.
    ISCC_Path : AnsiString;             //   InnoSetup.
    GlobalCachePath : AnsiString;       //     ().
    GlobalOutputPath : AnsiString;      //      .
    GlobalSourcePath : AnsiString;      //      .
    LocalOutputPath : AnsiString;       // (!) (!)      .
    LocalSourcePath : AnsiString;       // (!) (!)      .
    OldestVerFN : AnsiString;           //  AddUpdates      ( FN-)     iss-.

    //-...
    constructor Create; virtual;
    destructor Destroy; override;

    // ...
    function LoadConfigs() : Boolean;   //   Sconf  - *.vit  *.iss.     Iss.     -  sconf   [Main] ProjectVer=Project,              iss.
    procedure NewCurrentIssFromDef();       // CurrentIss   DefaultIss
    procedure AddAppInfo();        //  iss   .
    function AddVersionUpdate(var VitInfo : TVerInfoTxtFormat) : Boolean;    //  iss    .
    function AddVersionUpdate(const VerStr : AnsiString) : Boolean;   //  iss    .     - 4    +   -  0.1.0.10rc1
    function AddUpdates(const UpdVers : AnsiString; const SeparateBuilderMode : Boolean = false) : Boolean;   //  -   ,    UpdVers (    Sconf,   [IssBuilder].CreateSeparateUpds_UpdVers).     (SeparateBuilderMode) -   iss-     .
    function GenFullInstall() : Boolean;   //    Sconf,     .
    function GenSeparateUpdates() : Boolean;  //    Sconf,     .
    function GenCumulateUpdate() : Boolean;  //    Sconf,     .
    function ProcessSconf() : Boolean;   //  ,   Sconf -     .
    function BuildIssFile(const FileName : AnsiString) : Boolean;    //  iss   InnoSetup.

  end;

implementation

/////////////////////////////////////////////
//                        //
/////////////////////////////////////////////

function CompareVersionStrs(List: TStringList; Index1: Integer; Index2: Integer) : Integer;
//    TStringList  .
var
  Vit1, Vit2 : TVerInfoTxtFormat;
begin
  //.
  Result := 0;
  Vit1 := TVerInfoTxtFormat.Create;
  Vit2 := TVerInfoTxtFormat.Create;
  
  // .
  Vit1.ReadFromFNString(List.Strings[Index1]);
  Vit2.ReadFromFNString(List.Strings[Index2]);

  //.
  if Vit1.IIsNewerThenOldVit(Vit2) = true then begin
    Result := 1;
  end
  else if Vit1.IsSameVersion(Vit2) = false then begin
    Result := -1;
  end;
  
  // .
  Vit1.Free;
  Vit2.Free;
end;

/////////////////////////////////////////////
//             TIssBuilderLogic            //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

constructor TIssBuilderLogic.Create;
begin
  //  ...

  //    ...
  SconfFilePath := '';
  ErrorMessage := '';
  CurrentIssFileName := '';
  NoCacheWrite := false;
  ISCC_Path := '';
  GlobalCachePath := '';
  GlobalOutputPath := '';
  GlobalSourcePath := '';
  LocalOutputPath := '';
  LocalSourcePath := '';
  OldestVerFN := '';

  //   ...
  ProjectSconf := TSconfFormat.Create;
  ProjectVit := TVerInfoTxtFormat.Create;
  DefaultIss := TIssFormat.Create;
  CurrentIss := TIssFormat.Create;
  Registr := TRegistry.Create;
  CurrentFlist := TFlistFormat.Create;
end;

destructor TIssBuilderLogic.Destroy;
begin
  // 
  ProjectSconf.Free;
  ProjectVit.Free;
  DefaultIss.Free;
  CurrentIss.Free;
  Registr.Free;
  CurrentFlist.Free;

  //  
  inherited;
end;

//------------------------------------------//
//            ...           //
//------------------------------------------//

//------------------------------------------//
//             ...            //
//------------------------------------------//

function TIssBuilderLogic.LoadConfigs() : Boolean;
//   Sconf  - *.vit  *.iss.
//    Iss.
//    -  sconf   [Main] ProjectVer=Project,  
//           iss.
var
  TempStr : AnsiString;
  
begin
  //.
  Result := true;
  
  // .... ,    InnoSetup Compuler.
  //HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Inno Setup 5_is1 -       
  //    .
  Self.Registr.RootKey := HKEY_LOCAL_MACHINE;
  if Registr.OpenKey('\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Inno Setup 5_is1',false) = true then begin
    //  .     . ,      .
    if Registr.ValueExists('InstallLocation') = true then begin
      Self.ISCC_Path := AddLastSlash(Registr.ReadString('InstallLocation'))+ISSB_CompilerExecutableName;
    end;
    Registr.CloseKey;
    
    // -    .
    if FileExists(Self.ISCC_Path) = true then begin
      //  .
      Self.PostExternalMessage('InnoSetup 5 compiler found at path '+Self.ISCC_Path);
    end
    else begin
      // .
      Result := false;
      Self.ErrorMessage := 'InnoSetup 5 compiler at path '+Self.ISCC_Path+' is not exists!';
      Self.PostExternalMessage('Error! '+Self.ErrorMessage);
    end;
  end
  else begin
    //.    InnoSetup.
    Result := false;
    Self.ErrorMessage := 'InnoSetup 5 seems not to be installed.';
    Self.PostExternalMessage('Error! '+Self.ErrorMessage);
  end;
  
  //   -    *.vit .
  if Result = true then begin
    TempStr := AddLastSlash(Self.ProjectGlobalRoot)+Self.ProjectName+'.vit';   // *.vit .
    if FileExists(AddLastSlash(Self.ProjectGlobalRoot)+Self.ProjectName+'.vit') = true then begin
      //. .
      Self.PostExternalMessage('Reading file '+TempStr);
      if Self.ProjectVit.Load(TempStr) = true then begin
        //.
        Self.PostExternalMessage('Readed.');
      end
      else begin
        //  .
        Result := false;
        Self.ErrorMessage := 'VIT format error : '+Self.ProjectVit.ErrorMessage;
        Self.PostExternalMessage(Self.ErrorMessage);
      end;
    end
    else begin
      //  .
      Result := false;
      Self.ErrorMessage := 'Can''t find file '+TempStr;
      Self.PostExternalMessage('Error! '+Self.ErrorMessage);
    end;
  end;
  
  // -  *.iss.      .
  if Result = true then begin
    TempStr := Self.ProjectSconf.IssBuilderSection.DefaultIssName;
    //    .
    TempStr := ResolveLocalPath(TempStr,Self.ProjectGlobalRoot);
    //     - .
    if FileExists(TempStr) = true then begin
      //. .
      Self.PostExternalMessage('Reading file '+TempStr);
      if Self.DefaultIss.Load(TempStr) = true then begin
        //.
        Self.PostExternalMessage('Readed.');
      end
      else begin
        //  .
        Result := false;
        Self.ErrorMessage := 'VIT format error : '+Self.DefaultIss.ErrorMessage;
        Self.PostExternalMessage(Self.ErrorMessage);
      end;
    end
    else begin
      //  .
      Result := false;
      Self.ErrorMessage := 'Can''t find file '+TempStr;
      Self.PostExternalMessage('Error! '+Self.ErrorMessage);
    end;
  end;
  
end;

procedure TIssBuilderLogic.NewCurrentIssFromDef();
// CurrentIss   DefaultIss
begin
  Self.CurrentIss.MakNewFile();
  Self.CurrentIss.TextFil1.FileString := Self.DefaultIss.TextFil1.FileString;
end;

procedure TIssBuilderLogic.AddAppInfo();
//  iss   .
//var
  //TempStr : AnsiString;
begin
  //[Setup]
  //OutputDir
  Self.CurrentIss.WriteString('Setup','OutputDir',Self.LocalOutputPath);
  //SourceDir
  Self.CurrentIss.WriteString('Setup','SourceDir',Self.LocalSourcePath);
  //VersionInfoVersion
  Self.CurrentIss.WriteString('Setup','VersionInfoVersion',Self.ProjectVit.GenerateDotsVersionString());
  //VersionInfoTextVersion
  Self.CurrentIss.WriteString('Setup','VersionInfoTextVersion',Self.ProjectName+' '+Self.ProjectVit.GenerateShortVersionString_nosvn());
  //VersionInfoDescription
  Self.CurrentIss.WriteString('Setup','VersionInfoDescription',Self.ProjectName+' '+Self.ProjectVit.GenerateShortVersionString_nosvn());
  //AppVerName
  Self.CurrentIss.WriteString('Setup','AppVerName',Self.ProjectName+' '+Self.ProjectVit.GenerateShortVersionString_nosvn());
  
  //[Code]
  //AppName
  Self.CurrentIss.WriteCodeConstValue('AppName',''''+Self.ProjectName+'''');
  //AppVer
  Self.CurrentIss.WriteCodeConstValue('AppVer',''''+Self.ProjectVit.GenerateDotsVersionString()+'''');
  
end;

function TIssBuilderLogic.AddVersionUpdate(var VitInfo : TVerInfoTxtFormat) : Boolean;
//  iss    .
var
  OldIss : TIssFormat;
  ComponName, FuncName, OldFileName, NewFileName : AnsiString;
  
begin
  //.
  Result := true;
  OldIss := TIssFormat.Create;
  OldFileName := Self.GlobalCachePath+Self.ProjectName+'_'+VitInfo.GenerateFNVersionString();
  NewFileName := Self.GlobalCachePath+Self.ProjectName+'_'+Self.ProjectVit.GenerateFNVersionString();

  //.     -  ,    ,    .
  // ...
  ComponName := 'UpdFrom_'+VitInfo.GenerateFNVersionString_nostatus()+'_Main';
  FuncName := 'CheckUpdate_'+VitInfo.GenerateFNVersionString_nostatus();
  Self.CurrentIss.WriteComponent(ComponName,
    'Name: '+ComponName+'; Description:    '+VitInfo.GenerateShortVersionString_nosvn()+
    '; Flags: fixed; Check: '+FuncName+'; Types: StandardInstall');
    
  // - ...
  Self.CurrentIss.WriteCodeFunction(FuncName,'Function '+FuncName+'() : boolean;'+#13+#10+
    'begin'+#13+#10+'  if (CurrentVer = '''+VitInfo.GenerateDotsVersionString()+''') then Result := true;'+
    #13+#10+'end;');
    
  //   ...
  OldFileName := Self.GlobalCachePath+Self.ProjectName+'_'+VitInfo.GenerateFNVersionString();
  if FileExists(OldFileName+'.iss') = true then begin
    Self.CurrentIss.UpdateInfoToVerUpdate(OldFileName+'.iss',ComponName);
  end
  else begin
    Result := false;
    Self.ErrorMessage := 'Can''t load file '+OldFileName+'.iss';
    Self.PostExternalMessage('Error! '+Self.ErrorMessage);
  end;
  
  //   .
  if Result = true then begin
    Result := Self.CurrentIss.AddNewFilesFromFlists(OldFileName+'.flist',NewFileName+'.flist',ComponName);
    if Result = false then begin
      Self.ErrorMessage := 'Can''t compare flists '+OldFileName+'.flist and '+NewFileName+'.flist';
      Self.PostExternalMessage('Error! '+Self.ErrorMessage);
    end;
  end;
  
  // .
  OldIss.Free;
end;

function TIssBuilderLogic.AddVersionUpdate(const VerStr : AnsiString) : Boolean;
//  iss    .     - 4    +   -  0.1.0.10rc1
var
  TempVitInfo : TVerInfoTxtFormat;
  
begin
  // -  ,    -            .
  //.
  TempVitInfo := TVerInfoTxtFormat.Create;
  
  // ...
  Result := TempVitInfo.ReadFromFNString(VerStr);
  if Result = true then begin
    //   .
    Result := Self.AddVersionUpdate(TempVitInfo);
  end
  else begin
    Self.ErrorMessage := TempVitInfo.ErrorMessage;
    Self.PostExternalMessage('Error! '+Self.ErrorMessage);
  end;
  
  // .
  TempVitInfo.Free;
end;

function TIssBuilderLogic.AddUpdates(const UpdVers : AnsiString; const SeparateBuilderMode : Boolean = false) : Boolean;
//  -   ,    UpdVers (   
//Sconf,   [IssBuilder].CreateSeparateUpds_UpdVers).   
// (SeparateBuilderMode) -   iss-     .
var
  CurrIssBackUp, FindedName, TempStr, VersionsStr : AnsiString;
  VersionsList : TStringList;
  SearchRec : TSearchRec;
  I, UpdatesNum, J : Integer;
  TempVit1, TempVit2 : TVerInfoTxtFormat;
  Finded : Boolean;
  
begin
  //.
  Result := true;
  VersionsList := TStringList.Create;
  UpdatesNum := 0;
  TempVit1 := TVerInfoTxtFormat.Create;
  TempVit2 := TVerInfoTxtFormat.Create;
  
  //    ?
  if DirectoryExists(Self.GlobalCachePath) = false then begin
    Result := false;
    Self.ErrorMessage := 'Can''t find directory '+Self.GlobalCachePath;
    Self.PostExternalMessage('Error! '+Self.ErrorMessage);
  end;
  
  //  -    iss-.
  if (SeparateBuilderMode = true) and (Result = true) then begin
    CurrIssBackUp := Self.CurrentIss.TextFil1.FileString;
  end;
  
  //      .
  if Result = true then begin
    VersionsList.Clear;
    if FindFirst(AddLastSlash(Self.GlobalCachePath)+Self.ProjectName+'_*.flist', faAnyFile, SearchRec) = 0 then begin
      //   .
      repeat
        //         %).
        if (SearchRec.Name <> '.') and (SearchRec.Name <> '..') and (IsDirectory(AddLastSlash(Self.GlobalCachePath)+SearchRec.Name) = false) then begin
          //- .    ...
          FindedName := GetFileNameWithoutExtension(SearchRec.Name);
          //      .
          Delete(FindedName,1,Length(Self.ProjectName+'_'));
          //  ...
          if TempVit1.ReadFromFNString(FindedName) = true then begin;
            // -  ...   ...
            if Self.ProjectVit.IIsNewerThenOldVit(TempVit1) = true then begin
              //     -     .
              //    -   *.iss-    .
              if FileExists(AddLastSlash(Self.GlobalCachePath)+GetFileNameWithoutExtension(SearchRec.Name)+'.iss') = true then begin
                //    -  .
                VersionsList.Add(FindedName);
              end;
            end;
          end;
        end;
      until FindNext(SearchRec) <> 0;
      
      //  -  .
      if VersionsList.Count > 1 then begin
        VersionsList.CustomSort(@CompareVersionStrs);
      end;
    end
    else begin
      //   -      1 .
      Self.PostExternalMessage('Can''t found any version in directory '+Self.GlobalCachePath);
    end;
  end;
  
  //  .
  UpdatesNum := VersionsList.Count;    //  -  .
  if IsNumericStr(UpdVers) = true then begin
    //  StrToInt(UpdVers)  .
    if VersionsList.Count > StrToInt(UpdVers) then UpdatesNum := StrToInt(UpdVers);
  end
  else begin
    // All  From.
    if LowerCase(UpdVers) = 'all' then begin
      //All.     .
      //UpdatesNum := VersionsList.Count;    //  .
    end
    else begin
      //- From.
      if Length(UpdVers) > Length('From:') then begin
        //,  ...
        TempStr := UpdVers;
        if LowerCase(Parse(TempStr,':')) = 'from' then begin
          if TempVit1.ReadFromString(TempStr) = true then begin
            //   -        .
            UpdatesNum := 0;
            for I := 0 to VersionsList.Count-1 do begin
              // -       .
              TempVit2.ReadFromFNString(VersionsList.Strings[I]);
              if TempVit1.IIsNewerThenOldVit(TempVit2) = false then begin
                //     .
                UpdatesNum := UpdatesNum+1;
              end;;
            end;
          end;
        end;
      end;
    end;
  end;
  //.

  // -       iss.
  VersionsStr := '';   //        *.iss.
  if SeparateBuilderMode = false then begin
    //     -    .
    Self.PostExternalMessage('Adding updates:');
  end;
  For I := UpdatesNum-1 downto 0 do begin
    J := (VersionsList.Count-1)-I;   //  .
    //   ,       -   .
    if I = UpdatesNum-1 then begin
      Self.OldestVerFN := VersionsList.Strings[J];
    end;
    //  , ,  ...
    TempVit1.ReadFromFNString(VersionsList.Strings[J]);     //    -       .
    Result := Self.AddVersionUpdate(TempVit1);
    //     -    ,   iss-.
    if Result = true then begin
      if SeparateBuilderMode = false then begin
        //  .  .
        if Length(VersionsStr) > 0 then begin
          //    -  -   .
          VersionsStr := VersionsStr+';';
        end;
        VersionsStr := VersionsStr+TempVit1.GenerateDotsVersionString();
      end
      else begin
        //""  .
        //  iss-   ...
        Self.CurrentIss.WriteCodeConstValue('UpdateFrom',''''+TempVit1.GenerateDotsVersionString()+'''');
        //  Iss-...
        Self.CurrentIssFileName := AddLastSlash(Self.GlobalCachePath)+Self.ProjectName+'_'+TempVit1.GenerateFNVersionString()+'_to_'+Self.ProjectVit.GenerateFNVersionString()+'.iss';
        //  Output...
        Self.CurrentIss.WriteString('Setup','OutputBaseFilename',GetFileNameWithoutExtension(Self.CurrentIssFileName));
        // Iss-.
        Result := Self.CurrentIss.Save(Self.CurrentIssFileName);
        if Result = true then begin
          Self.PostExternalMessage('Saved file '+Self.CurrentIssFileName);
          //  Iss-.
          Result := Self.BuildIssFile(Self.CurrentIssFileName);
          if Self.NoCacheWrite = true then begin
            //      -   iss-
            if DeleteFile(Self.CurrentIssFileName) = true then begin
              Self.PostExternalMessage('Deleted file '+Self.CurrentIssFileName);
            end
            else begin
              Self.PostExternalMessage('Achtung!!! Can''t delete file '+Self.CurrentIssFileName);
            end;
          end;
        end
        else begin
          // .
          Self.ErrorMessage := 'Can''t save file '+Self.CurrentIssFileName;
          Self.PostExternalMessage('Error! '+Self.ErrorMessage);
        end;
        //   -    iss-.
        Self.CurrentIss.TextFil1.FileString := CurrIssBackUp;
      end;
    end
    else begin
      //   .       AddVersionUpdate.
    end;
    
    //   -      -     .
    if (Result = true) and (SeparateBuilderMode = false) then begin
      Writeln('         '+VersionsList.Strings[J]);
    end;
    
    //     - .
    if Result = false then Break;
  end;
  
  if (Result = true) and (SeparateBuilderMode = false) then begin
    //     -     .
    Self.CurrentIss.WriteCodeConstValue('UpdateFrom',''''+VersionsStr+'''');
    //     .
    Writeln('Updates added.');
  end;
  
  // .
  VersionsList.Free;
  TempVit1.Free;
  TempVit2.Free;
end;

function TIssBuilderLogic.GenFullInstall() : Boolean;
//    Sconf,     .
begin
  //.
  Result := true;
  
  // Iss-...
  Self.NewCurrentIssFromDef();
  Self.CurrentIssFileName := AddLastSlash(Self.GlobalCachePath)+Self.ProjectName+'_'+Self.ProjectVit.GenerateFNVersionString()+'.iss';
  Self.AddAppInfo();
  
  //  .
  Self.CurrentIss.WriteCodeConstValue('NoFullInstall','false');
  Self.CurrentIss.WriteString('Setup','OutputBaseFilename',Self.ProjectName+'_'+Self.ProjectVit.GenerateFNVersionString());
  if Self.ProjectSconf.IssBuilderSection.CreateFull_AllowUpdate = true then begin
    //    -     .
    Self.CurrentIss.WriteCodeConstValue('NoUpdate','false');
    //        -  
    //    .
    Self.CurrentIss.WriteCodeConstValue('UpdateFrom','''''');
  end
  else begin
    //    -  .
    Self.CurrentIss.WriteCodeConstValue('NoUpdate','true');
  end;
  
  //   .
  // ...
  Self.CurrentIss.WriteComponent('CurrVersion_Main','Name: CurrVersion_Main; Description:    ; Flags: fixed; Check: CheckDoNewInstall; Types: StandardInstall');
  //    ...
  Self.CurrentIss.AddFilesFromFlist(Self.CurrentFlist,'CurrVersion_Main');
  
  // ...
  Self.AddUpdates(Self.ProjectSconf.IssBuilderSection.CreateFull_UpdVers);
  
  // Iss-.
  Result := Self.CurrentIss.Save(Self.CurrentIssFileName);
  if Result = true then begin
    Self.PostExternalMessage('Saved file '+Self.CurrentIssFileName);
    //  Iss-.
    Result := Self.BuildIssFile(Self.CurrentIssFileName);
    if Self.NoCacheWrite = true then begin
      //      -   iss-
      if DeleteFile(Self.CurrentIssFileName) = true then begin
        Self.PostExternalMessage('Deleted file '+Self.CurrentIssFileName);
      end
      else begin
        Self.PostExternalMessage('Achtung!!! Can''t delete file '+Self.CurrentIssFileName);
      end;
    end;
  end
  else begin
    // .
    Self.ErrorMessage := 'Can''t save file '+Self.CurrentIssFileName;
    Self.PostExternalMessage('Error! '+Self.ErrorMessage);
  end;
end;

function TIssBuilderLogic.GenSeparateUpdates() : Boolean;
//    Sconf,     .
begin
  //.
  Result := true;

  // Iss-...
  Self.NewCurrentIssFromDef();

  Self.AddAppInfo();

  //  .
  Self.CurrentIss.WriteCodeConstValue('NoFullInstall','true');
  Self.CurrentIss.WriteCodeConstValue('NoUpdate','false');
  //        -  
  //    .
  Self.CurrentIss.WriteCodeConstValue('UpdateFrom','''''');
  
  //      .
  Self.AddUpdates(Self.ProjectSconf.IssBuilderSection.CreateSeparateUpds_UpdVers,true);
end;

function TIssBuilderLogic.GenCumulateUpdate() : Boolean;
//    Sconf,     .
begin
  //.
  Result := true;

  // Iss-...
  Self.NewCurrentIssFromDef();

  Self.AddAppInfo();
  
  //  .
  Self.CurrentIss.WriteCodeConstValue('NoFullInstall','true');
  Self.CurrentIss.WriteCodeConstValue('NoUpdate','false');
  //        -  
  //    .
  Self.CurrentIss.WriteCodeConstValue('UpdateFrom','''''');
  
  // ...
  Self.AddUpdates(Self.ProjectSconf.IssBuilderSection.CreateCumulateUpdate_UpdVers);
  
  // -    - ?
  if Length(Self.CurrentIss.ReadCodeConstValue('UpdateFrom')) > 2 then begin
    //, -    -     .
    //  .
    Self.CurrentIssFileName := AddLastSlash(Self.GlobalCachePath)+Self.ProjectName+'_'+Self.OldestVerFN+'_to_'+Self.ProjectVit.GenerateFNVersionString()+'_CumulateUpdate.iss';
    Self.CurrentIss.WriteString('Setup','OutputBaseFilename',Self.ProjectName+'_'+Self.OldestVerFN+'_to_'+Self.ProjectVit.GenerateFNVersionString()+'_CumulateUpdate');

    // Iss-.
    Result := Self.CurrentIss.Save(Self.CurrentIssFileName);
    if Result = true then begin
      Self.PostExternalMessage('Saved file '+Self.CurrentIssFileName);
      //  Iss-.
      Result := Self.BuildIssFile(Self.CurrentIssFileName);
      if Self.NoCacheWrite = true then begin
        //      -   iss-
        if DeleteFile(Self.CurrentIssFileName) = true then begin
          Self.PostExternalMessage('Deleted file '+Self.CurrentIssFileName);
        end
        else begin
          Self.PostExternalMessage('Achtung!!! Can''t delete file '+Self.CurrentIssFileName);
        end;
      end;
    end
    else begin
      // .
      Self.ErrorMessage := 'Can''t save file '+Self.CurrentIssFileName;
      Self.PostExternalMessage('Error! '+Self.ErrorMessage);
    end;
  end
  else begin
    // -      -     .
    Self.PostExternalMessage('Nothing to update - cumulative update is not needable.');
  end;

end;

function TIssBuilderLogic.ProcessSconf() : Boolean;
//  ,   Sconf -     .
var
  TempStr : AnsiString;
  
begin
  //.
  Result := true;
  
  //  ,    .
  Self.ProjectName := GetFileNameWithoutExtension(Self.SconfFilePath);
  Self.ProjectLocalRoot := AddLastSlash(Self.ProjectSconf.MainSection.ProjectRootDir);
  Self.ProjectGlobalRoot := ResolveLocalPath(Self.ProjectLocalRoot,ExtractFilePath(Self.SconfFilePath));
  
  // .
  Result := Self.LoadConfigs();
  
  //?
  if Result = true then begin
    //,  .
    //  .
    Self.GlobalCachePath := ResolveLocalPath(Self.ProjectSconf.IssBuilderSection.VersionsCacheDir,Self.ProjectGlobalRoot);
    Self.GlobalOutputPath := ResolveLocalPath(Self.ProjectSconf.IssBuilderSection.OutputDir,Self.ProjectGlobalRoot);
    Self.GlobalSourcePath := ResolveLocalPath(Self.ProjectSconf.IssBuilderSection.FilesDir,Self.ProjectGlobalRoot);
    Self.LocalOutputPath := GetLocalPath(Self.GlobalCachePath,Self.GlobalOutputPath);
    Self.LocalSourcePath := GetLocalPath(Self.GlobalCachePath,Self.GlobalSourcePath);
    
    // -     output  cache.    - .
    if DirectoryExists(Self.GlobalCachePath) = false then begin
      Result := ForceDirectories(Self.GlobalCachePath);
      if Result = true then begin
        //   .
        Self.PostExternalMessage('Created directory '+Self.GlobalCachePath);
      end
      else begin
        //  .
        Self.ErrorMessage := 'Can''t reate directory '+Self.GlobalCachePath;
        Self.PostExternalMessage('Error! '+Self.ErrorMessage);
      end;
    end;
    if (DirectoryExists(Self.GlobalOutputPath) = false) and (Result = true) then begin
      Result := ForceDirectories(Self.GlobalOutputPath);
      if Result = true then begin
        //   .
        Self.PostExternalMessage('Created directory '+Self.GlobalOutputPath);
      end
      else begin
        //  .
        Self.ErrorMessage := 'Can''t reate directory '+Self.GlobalOutputPath;
        Self.PostExternalMessage('Error! '+Self.ErrorMessage);
      end;
    end;
    
    //  flist-   .
    if Result = true then begin
      Self.PostExternalMessage('Creating distributive file list...');
      Self.CurrentFlist.Clear();
      Self.CurrentFlist.GlobalRoot := Self.GlobalSourcePath;
      Self.CurrentFlist.PathsAreAbsolute := false;
      Self.CurrentFlist.AddSize := true;
      Self.CurrentFlist.AddCRC32 := true;
      Result := Self.CurrentFlist.AddRealDirectory(Self.GlobalSourcePath,true);
      if Result = true then begin
        if Self.NoCacheWrite = false then begin
          //. .
          TempStr := AddLastSlash(Self.GlobalCachePath)+Self.ProjectName+'_'+Self.ProjectVit.GenerateFNVersionString()+'.flist';
          Result := Self.CurrentFlist.Save(TempStr);
          if Result = true then begin
            //.
            Self.PostExternalMessage('Saved file '+TempStr);
            //         .
            if Self.ProjectSconf.IssBuilderSection.SaveToCache = true then begin
              // .      - .
              TempStr := AddLastSlash(Self.GlobalCachePath)+Self.ProjectName+'_'+Self.ProjectVit.GenerateFNVersionString();
              if DirectoryExists(TempStr) = true then begin
                Self.PostExternalMessage('Removing old files cache from '+TempStr);
                DeleteDirectory(TempStr,true);
                Self.PostExternalMessage('Done.');
              end;
              // .
              Self.PostExternalMessage('Saving files cache to '+TempStr);
              CopyDirectory(Self.GlobalSourcePath, TempStr, true, true);
              Self.PostExternalMessage('Done.');
            end;
          end
          else begin
            // .
            Self.ErrorMessage := 'Can''t save file '+TempStr;
            Self.PostExternalMessage('Error! '+Self.ErrorMessage);
          end;
        end;
      end
      else begin
        //  .
        Self.ErrorMessage := 'Can''t create file list for directory '+Self.GlobalSourcePath+'. Error message was: '+Self.CurrentFlist.ErrorMessage;
        Self.PostExternalMessage('Error! '+Self.ErrorMessage);
      end;
    end;

    // -    ,   Sconf  .
    if Result = true then begin
      // .
      if Self.ProjectSconf.IssBuilderSection.CreateFull = true then begin
        Result := Self.GenFullInstall();
      end;

      // .
      if (Self.ProjectSconf.IssBuilderSection.CreateSeparateUpds = true) and (Result = true) then begin
        Result := Result and Self.GenSeparateUpdates();
      end;

      //  .
      if (Self.ProjectSconf.IssBuilderSection.CreateCumulateUpdate = true) and (Result = true) then begin
        Result := Result and Self.GenCumulateUpdate();
      end;
    end;
    
  end;
end;

function TIssBuilderLogic.BuildIssFile(const FileName : AnsiString) : Boolean;
//  iss   InnoSetup.
var
  IssCompiler : TProcess;
  
begin
  //.
  Result := true;
  IssCompiler := TProcess.Create(Application);
  
  if FileExists(FileName) = true then begin
    Self.PostExternalMessage('Compilling file '+FileName);
    // ...
    IssCompiler.CommandLine := Self.ISCC_Path+' '+FileName;
    IssCompiler.CurrentDirectory := Self.GlobalCachePath;
    IssCompiler.Options := [poNoConsole, poWaitOnExit];
    // .
    IssCompiler.Execute;
    if IssCompiler.ExitStatus <> 0 then begin
      //  .
      Result := false;
      Self.ErrorMessage := 'Inno Setup compiler returned ExitStatus='+IntToStr(IssCompiler.ExitStatus)+' - seems that it was any error! Try to compile '+FileName+' manually to check for errors in script.';
      Self.PostExternalMessage('Error! '+Self.ErrorMessage);
    end
    else begin
      Self.PostExternalMessage('Compilled succesfully!');
    end;
  end
  else begin
    // .
    Result := false;
    Self.ErrorMessage := 'File not exists: '+FileName;
    Self.PostExternalMessage('Error! '+Self.ErrorMessage);
  end;;
  
  // .
  IssCompiler.Free;
end;

end.

