///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////
//            UnInstaller            //
////////////////////////////////////////////////////////

unit UnInstallerServiceClass;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Registry, Forms, ExtraFunctionsLcl, GGBuildToolsShared, GGConsUtilServiceClass, ExtraFileUtilsLCL;
  
const
  GGBT_UnInstallerAppName = 'UnInstaller';
  
type

  TUnInstallerServiceClass = class (TGGConsUtilServiceClass)      //      
  private
  protected
  public
    //
    Registr : TRegistry;               //      .
    InstalledTools : RInstalledTools;  //   

    //-...
    constructor Create; override;
    destructor Destroy; override;

    // ...
    procedure DoWork;       override;  //   %).
    procedure ShowHelp;     override;  // .
    procedure UnInstallIt;             // .

  end;
  
var
  UnInstServClass : TUnInstallerServiceClass;

implementation

/////////////////////////////////////////////
//         TInstallerServiceClass          //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

constructor TUnInstallerServiceClass.Create;
begin
  //  ...
  InstalledTools.IssBuilder_installed := false;
  InstalledTools.StarterModBuilder_installed := false;
  InstalledTools.StarterModSynchronizator_installed := false;
  InstalledTools.VerRevUpdater_installed := false;

  //   ...
  Registr := TRegistry.Create;
  
  //  .
  Self.GGConsAppName := GGBT_UnInstallerAppName;

  //  ...
  inherited;
end;

destructor TUnInstallerServiceClass.Destroy;
begin
  // 
  Registr.Free;

  //  
  inherited;
end;

//-----------------------------------------//
//            ...              //
//-----------------------------------------//

procedure TUnInstallerServiceClass.DoWork;
//   %).
begin
  //     -    ProcessFurther := false
  //    .

  //  ...
  inherited;

  //       -    ....
  if ProcessFurther = true then begin
    //  
    ProcessFurther := false;
    //....
    UnInstallIt;
  end;
  
end;

procedure TUnInstallerServiceClass.ShowHelp;
// .
begin
  //  ...
  inherited;
  
  // .
  Writeln();
end;

procedure TUnInstallerServiceClass.UnInstallIt;
// .
var
  NeedUnInstall, DelExePath, DelRegPath : boolean;
  RegPath, ExePath, Answer : string;
  
begin
  //   :    .    - 
  //   . .    path.      
  //  -  ,   path  1   -   ,  
  //    ,    y () -  silent.  
  //         %).
  
  // .
  NeedUnInstall := false;   //  -     %).
  ExePath := ExtractFilePath(Application.ExeName)+'\bin';   //    GGBuildTools.
  DelExePath := false;
  DelRegPath := true;
  
  //  ...
  ShowVersion();
  
  //   ...
  Writeln('Gathering infirmation...');
  Writeln('');
  
  //,   ...
  Registr.RootKey := HKEY_CURRENT_USER;
  if Registr.OpenKey('\Software\Gipat Group\'+AllProjectName,false) = true then begin
  //if Registr.KeyExists('\Software\Gipat Group\'+AllProjectName,false) = true then begin    //#LCL-BUG
    //Registr.OpenKey('\Software\Gipat Group\'+AllProjectName,false);                        //#LCL-BUG

    //  ...     %).
    RegPath := Registr.ReadString('path')+'\bin';
    Registr.CloseKey;
    
    //   (   )  -       path-.
    if UpperCase(DelLastSlash(ExePath)) <> UpperCase(DelLastSlash(RegPath)) then begin
      if (EnvPathExists(ExePath,true,true) = true) and (EnvPathExists(RegPath,true,true) = true) then begin
        //      path. ,    ...
        if DoSilent = false then begin
          //.
          Writeln('');
          Write('Delete "'+ExePath+'" from path? (y/n) > ');
          Readln(Answer);
          if UpperCase(Answer) = 'Y' then begin
            DelExePath := true;
          end
          else begin
            DelExePath := false;
          end;
          Writeln('');
          
          Writeln('');
          Write('Delete "'+RegPath+'" from path? (y/n) > ');
          Readln(Answer);
          if UpperCase(Answer) = 'Y' then begin
            DelRegPath := true;
          end
          else begin
            DelRegPath := false;
          end;
          Writeln('');
        end
        else begin
          //  .
          DelExePath := true;
          DelRegPath := true;
          Writeln('');
          Writeln('I''ll delete "'+ExePath+'" automatically due to silent mode.');
          Writeln('I''ll delete "'+RegPath+'" automatically due to silent mode.');
          Writeln('');
        end;
      end;
    end;
    
    //.   . ,      silent
    //     silent.
    Writeln('');
    if DoSilent = false then begin
      Write('Ok, i''m ready to uninstall '+AllProjectName+'. Continue? (y/n) > ');
      Readln(Answer);
      if UpperCase(Answer) = 'Y' then begin
        NeedUnInstall := true;
        Writeln('Ok, uninstalling.');
      end
      else begin
        NeedUnInstall := false;
        Writeln('Ok, canceled.');
      end;
    end
    else begin
      NeedUnInstall := true;
      Writeln('Ok, i''m ready to uninstall '+AllProjectName+'. Uninstalling!');
    end;
    Writeln('');
    
  end
  else begin
    //   . ...
    Writeln(AllProjectName+' is not installed. UnInstallation canceled.');
    Writeln('');
  end;
  
  //,  ,    .
  if NeedUnInstall = true then begin
    //,     .
    Writeln('Uninstalling started...');

    // path...
    if DelExePath = true then begin
      EnvPathRemove(ExePath, true, true);
      Writeln('Deleted "'+ExePath+'" from path.');
    end;
    if DelRegPath = true then begin
      EnvPathRemove(RegPath, true, true);
      Writeln('Deleted "'+RegPath+'" from path.');
    end;
    
    //  ...
    Registr.DeleteKey('\Software\Gipat Group\'+AllProjectName);
    Writeln('Deleted HKEY_CURRENT_USER\Software\Gipat Group\'+AllProjectName+' from registry.');

    Writeln('');
    Writeln('UnInstallation completed succesfully!');
    Writeln('');
    
  end;
end;

/////////////////////////////////////////////
initialization

// %).  ,   ...     %)
begin
  UnInstServClass := TUnInstallerServiceClass.Create;
end;

finalization
//  ...
begin
  UnInstServClass.Free;
end;

end.

