///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
//            (..            //
//          )   VerRevUpdater        //
///////////////////////////////////////////////////////////////

unit VerRevUpdLogic;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, SconfFormat, VerInfoTxtFormat, LCLTextFileInString, ExtraFileUtilsLCL,
  Dialogs, ExtraFunctionsLCL, SvnInfoReader, LCLAnIniFile;

type
  FPostExternalMessage = procedure(const TextMessage : AnsiString) of object;    //       .

  TVerRevUpdaterLogic = class     //,    ,     
  private
  protected
    //.
    SconfFilePath : AnsiString;         //    sconf-.
    ProjectName : AnsiString;           // .
    ProjectLocalRoot : AnsiString;      //   .
    ProjectGlobalRoot : AnsiString;     //   .
    ProjectSconf : TSconfFormat;        //   sconf .
    ProjectVit : TVerInfoTxtFormat;     //  \ *.vit
    CodeFile : TTextFileInString;       // , ""   *.pas  *.h  - .
    
    // .
    function UpdateVerInfoTxtOnlyProj() : Boolean;      //   VerInfoTxtOnly.
    function UpdatePascalCodeProj() : Boolean;          //   PascalCode.
    function UpdateCppCodeProj() : Boolean;             //   CppCode.
  public
    //.
    ErrorMessage : AnsiString;                          //  .
    PostExternalMessage : FPostExternalMessage;         //,     .
    
    //-...
    constructor Create; virtual;
    destructor Destroy; override;
    
    // ...
    function UpdateProjVer(const FullSconfFilePath : AnsiString) : Boolean;    //       *.sconf-.
  end;

implementation

/////////////////////////////////////////////
//           TVerRevUpdaterLogic           //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

constructor TVerRevUpdaterLogic.Create;
begin
  //  ...
  
  //    ...
  SconfFilePath := '';
  ErrorMessage := '';

  //   ...
  ProjectSconf := TSconfFormat.Create;
  ProjectVit := TVerInfoTxtFormat.Create;
  CodeFile := TTextFileInString.Create;
end;

destructor TVerRevUpdaterLogic.Destroy;
begin
  // 
  ProjectSconf.Free;
  ProjectVit.Free;
  CodeFile.Free;

  //  
  inherited;
end;

//------------------------------------------//
//            ...           //
//------------------------------------------//

function TVerRevUpdaterLogic.UpdateVerInfoTxtOnlyProj() : Boolean;
//   VerInfoTxtOnly.
begin
  //  -   *.vit-    .
  Result := Self.ProjectVit.Save(Self.ProjectLocalRoot+Self.ProjectName+'.vit');
  if Result = true then begin
    Self.PostExternalMessage('Saved file '+Self.ProjectLocalRoot+Self.ProjectName+'.vit');
  end
  else begin
    //     .
    Self.PostExternalMessage('Cant save file '+Self.ProjectLocalRoot+Self.ProjectName+'.vit');
    Self.ErrorMessage := 'Cant save file '+Self.ProjectLocalRoot+Self.ProjectName+'.vit';
  end;
end;

function TVerRevUpdaterLogic.UpdatePascalCodeProj() : Boolean;
//   PascalCode.
var
  CodeTextFile : TTextFileInString;
begin
  //  vit.
  Result := Self.UpdateVerInfoTxtOnlyProj();
  
  //   - .
  if Result = true then begin
    //           % ))).
    CodeTextFile := TTextFileInString.Create;
    CodeTextFile.MakNewFile;
    
    //  % )).
    CodeTextFile.AddStr('/////////////////////////////////////////////////////////////////////////////');
    CodeTextFile.AddStr('//         This file was generated automaticallyby VerRevUpdater.          //');
    CodeTextFile.AddStr('//          It includes a version number for this application.             //');
    CodeTextFile.AddStr('//    Please, don''t modify it manually - it''ll be recreated next build.    //');
    CodeTextFile.AddStr('/////////////////////////////////////////////////////////////////////////////');
    CodeTextFile.AddStr('unit '+Self.ProjectName+'_ver;');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('interface');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('const');
    CodeTextFile.AddStr('  '+Self.ProjectName+'_Ver1 = '+IntToStr(Self.ProjectVit.Ver1)+';');
    CodeTextFile.AddStr('  '+Self.ProjectName+'_Ver2 = '+IntToStr(Self.ProjectVit.Ver2)+';');
    CodeTextFile.AddStr('  '+Self.ProjectName+'_Ver3 = '+IntToStr(Self.ProjectVit.Ver3)+';');
    CodeTextFile.AddStr('  '+Self.ProjectName+'_Ver4 = '+IntToStr(Self.ProjectVit.Ver4)+';');
    CodeTextFile.AddStr('  '+Self.ProjectName+'_Status = '+IntToStr(Self.ProjectVit.Status)+';');
    CodeTextFile.AddStr('  '+Self.ProjectName+'_RCNum = '+IntToStr(Self.ProjectVit.RCNum)+';');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('implementation');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('end.');

    //.
    CodeTextFile.Save(Self.ProjectLocalRoot+Self.ProjectName+'_ver.pas');
    Self.PostExternalMessage('Saved file '+Self.ProjectLocalRoot+Self.ProjectName+'_ver.pas');
    
    //   .
    CodeTextFile.Free;
  end;
end;

function TVerRevUpdaterLogic.UpdateCppCodeProj() : Boolean;
//   CppCode.
var
  CodeTextFile : TTextFileInString;
begin
  //  vit.
  Result := Self.UpdateVerInfoTxtOnlyProj();

  //   - .
  if Result = true then begin
    //           % ))).
    CodeTextFile := TTextFileInString.Create;
    CodeTextFile.MakNewFile;

    //  % )).
    CodeTextFile.AddStr('/////////////////////////////////////////////////////////////////////////////');
    CodeTextFile.AddStr('//         This file was generated automaticallyby VerRevUpdater.          //');
    CodeTextFile.AddStr('//          It includes a version number for this application.             //');
    CodeTextFile.AddStr('//    Please, don''t modify it manually - it''ll be recreated next build.    //');
    CodeTextFile.AddStr('/////////////////////////////////////////////////////////////////////////////');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('#ifndef '+UpperCase(Self.ProjectName)+'_VER_H_INCLUDED');
    CodeTextFile.AddStr('#define '+UpperCase(Self.ProjectName)+'_VER_H_INCLUDED');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('const int '+Self.ProjectName+'_Ver1 = '+IntToStr(Self.ProjectVit.Ver1)+';');
    CodeTextFile.AddStr('const int '+Self.ProjectName+'_Ver2 = '+IntToStr(Self.ProjectVit.Ver2)+';');
    CodeTextFile.AddStr('const int '+Self.ProjectName+'_Ver3 = '+IntToStr(Self.ProjectVit.Ver3)+';');
    CodeTextFile.AddStr('const int '+Self.ProjectName+'_Ver4 = '+IntToStr(Self.ProjectVit.Ver4)+';');
    CodeTextFile.AddStr('const int '+Self.ProjectName+'_Status = '+IntToStr(Self.ProjectVit.Status)+';');
    CodeTextFile.AddStr('const int '+Self.ProjectName+'_RCNum = '+IntToStr(Self.ProjectVit.RCNum)+';');
    CodeTextFile.AddStr('');
    CodeTextFile.AddStr('#endif // '+UpperCase(Self.ProjectName)+'_VER_H_INCLUDED');

    //.
    CodeTextFile.Save(Self.ProjectLocalRoot+Self.ProjectName+'_ver.h');
    Self.PostExternalMessage('Saved file '+Self.ProjectLocalRoot+Self.ProjectName+'_ver.h');

    //   .
    CodeTextFile.Free;
  end;
end;

//------------------------------------------//
//             ...            //
//------------------------------------------//

function TVerRevUpdaterLogic.UpdateProjVer(const FullSconfFilePath : AnsiString) : Boolean;
//       *.sconf-.
var
  SvnReader : TSvnInfoReader;
  TempVit : TVerInfoTxtFormat;
  
begin
  //.
  Result := true;
  
  // Sconf.
  Self.PostExternalMessage('Reading sconf from '+FullSconfFilePath);
  Result := Self.ProjectSconf.Load(FullSconfFilePath);
  if Result = true then begin
    //.  -     -       
    //  "Project"
    Self.PostExternalMessage('Readed.');
    if LowerCase(Self.ProjectSconf.MainSection.ProjectVer) = 'project' then begin
      //. .   ,    .
      Self.ProjectName := GetFileNameWithoutExtension(FullSconfFilePath);
      Self.ProjectLocalRoot := AddLastSlash(Self.ProjectSconf.MainSection.ProjectRootDir);
      Self.ProjectGlobalRoot := ResolveLocalPath(Self.ProjectLocalRoot,ExtractFilePath(FullSconfFilePath));
      // -  *.vit-.
      Self.PostExternalMessage('Reading version from'+Self.ProjectLocalRoot+Self.ProjectName+'_dev.vit');
      Result := Self.ProjectVit.Load(Self.ProjectLocalRoot+Self.ProjectName+'_dev.vit');
      //to be continued -   -     ,  if-  Result %).
    end
    else begin
      Self.PostExternalMessage('Project don''t using VerRevUpdating. Nothing to do.');
      Exit;
    end;
  end
  else begin
    //   .
    Self.ErrorMessage := 'Cant load '+FullSconfFilePath;
  end;
  
  //  vit-?
  if Result = true then begin
    //.
    Self.PostExternalMessage('Readed. Version is '+Self.ProjectVit.GenerateVersionString());
    //   -    SVN.
    if Self.ProjectSconf.MainSection.UseSvn = true then begin
      // .
      Self.PostExternalMessage('Reading SVN revision.');
      SvnReader := TSvnInfoReader.Create;
      Self.ProjectVit.Ver4 := SvnReader.GetSvnDirRevision(Self.ProjectLocalRoot);
      //  0 -    -   .
      if Self.ProjectVit.Ver4 = 0 then begin
        Self.PostExternalMessage(SvnReader.ErrorMessage);
        //       [ ].vit    .
        if FileExists(Self.ProjectLocalRoot+Self.ProjectName+'.vit') = true then begin
          //,  . .
          TempVit := TVerInfoTxtFormat.Create;
          if TempVit.Load(Self.ProjectLocalRoot+Self.ProjectName+'.vit') = true then begin
            Self.ProjectVit.Ver4 := TempVit.Ver4;
            Self.PostExternalMessage('Revision restored from old '+Self.ProjectLocalRoot+Self.ProjectName+'.vit file.');
          end;
          TempVit.Free;
        end;
      end;
      Self.PostExternalMessage('Revision is '+IntToStr(Self.ProjectVit.Ver4)+'. Version will be '+Self.ProjectVit.GenerateVersionString());
      //   .
      SvnReader.Free;
    end;
    
    //.  -       ,      .
    if Self.ProjectSconf.MainSection.ProjectType = ScF_ProjectType_VerInfoTxtOnly then begin
      // vit.
      Self.PostExternalMessage('Updating VerInfoTxtOnly project.');
      Result := Self.UpdateVerInfoTxtOnlyProj();
    end
    else if Self.ProjectSconf.MainSection.ProjectType = ScF_ProjectType_PascalCode then begin
      //.
      Self.PostExternalMessage('Updating PascalCode project.');
      Result := Self.UpdatePascalCodeProj();
    end
    else if Self.ProjectSconf.MainSection.ProjectType = ScF_ProjectType_CppCode then begin
      // %).
      Self.PostExternalMessage('Updating CppCode project.');
      Result := Self.UpdateCppCodeProj();
    end;
  end;
end;

end.

